import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Ant is a creature looking for food in the Arena. It moves randomly until it finds food
 * which it will take home.
 * 
 * @author Pavel Soukup 
 * @version v1
 */
public class Ant extends Actor
{
    private Anthill home;
    private boolean hasFood = false;
    private boolean smellsFood = false;
    private static int SMELL_DISTANCE = 45;
    private int smellDistance = SMELL_DISTANCE;
    private int stepSize = 2;
    private int saturation = 700+Greenfoot.getRandomNumber(800);
    private int distanceWalked;

    /**
     * Constructor for creating an ant with an assigned anthill and random orientation
     */
    public Ant(Anthill anthill)
    {
        home = anthill;
        setRotation(Greenfoot.getRandomNumber(360));
    }

    /**
     * Act - do whatever the Ant wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        checkHome();
        if(hasFood){
            goTowards(home);
        } else {
            Actor food = smellFood();
            if(food!=null){
                goTowards(food);

                Actor closeFood;
                closeFood = getOneObjectAtOffset(0,0,Flower.class);

                if(closeFood!=null){
                    pickUpFood((Flower)closeFood);
                }

            } else {
                if(atWorldEdge()){
                    turn(180);
                } else {
                    turn(10-Greenfoot.getRandomNumber(21)); //otoc nahodne v rozmezi -10 az 10
                }
                move(stepSize);
            }
        }

        //ants have to eat
        saturation--;
        distanceWalked++;
        //checkAntEaterLocation(); //blbne!!!
        if(saturation<0){
            home.setCurrentPop(home.getCurrentPop()-1);
            getWorld().removeObject(this);
        }
    }

    /**
     * Return TRUE if ant is at the worlds edge
     */
    public boolean atWorldEdge()
    {
        if(getX() < 10 || getX() > getWorld().getWidth() - 10){ return true;}
        if(getY() < 10 || getY() > getWorld().getHeight() - 10){ return true;}
        return false;
    }

    /**
     * An example of a method - replace this comment with your own
     *
     */
    public Actor smellFood()
    {
        Actor closest = null; //empty closest food reference
        if(home.getFoodCount()>0){
            if(distanceWalked+100>saturation){
                goTowards(home);}}
                if(!getObjectsInRange(smellDistance, Flower.class).isEmpty()){
                    for(Object obj: getObjectsInRange(smellDistance, Flower.class)){
                        Actor food = (Actor) obj;
                        int foodDist = (int) Math.sqrt((Math.pow(food.getX()-getX(),2)+Math.pow(food.getY()-getY(),2)));

                        if(closest == null || foodDist < smellDistance){ //if distance is smaller than smellDistance OR there is NONE
                            closest = food;
                            smellDistance = foodDist; //decrease smell distance for next time to avoid smelling multiple sources
                        }
                    }
                }
            
            return(closest);
        }
        
    

    /**
     * Makes ant go in a direction.
     */
    public void goTowards(Actor target)
    {
        int x = target.getX();
        int y = target.getY();
        if(getX()==x&&getY()==y){
            //rest, you are there
        } else {
            turnTowards(x,y);
            move(stepSize);
        }
    }

    /**
     * An example of a method - replace this comment with your own
     */
    public void pickUpFood(Flower food)
    {
        hasFood = true;
        food.takeFood();
        setImage("ant-with-food.png");
    }

    /**
     * Checks if ant is at home
     */
    public void checkHome()
    {
        if(Math.abs(getX() - home.getX()) <= stepSize && 
        Math.abs(getY() - home.getY()) <= stepSize){
            distanceWalked = 0;
            smellDistance = SMELL_DISTANCE;
            if(hasFood == true){
                dropFood();
            }
            if(home.getFoodCount()>0){
                if(saturation<2000){
                    home.setFoodCount(home.getFoodCount()-1);
                    saturation = saturation+700;
                }
            }
        }
    }

    /**
     * Drops food
     */
    public void dropFood()
    {
        hasFood = false;
        setImage("ant.png");
        home.setFoodCount(home.getFoodCount()+1);
    }

    /**
     * Checks if AntEater is close
     
    public void checkAntEaterLocation()
    {
        //getWorld().getObjects(AntEater.class).get(0);
        Actor hit;
        hit = getOneIntersectingObject(AntEater.class);
        if(hit != null){
            if(Greenfoot.getRandomNumber(8)>6){getWorld().removeObject(this);
            home.setCurrentPop(home.getCurrentPop()-1);}
        }
    }
    */
}
