import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Bee is a creature producing food in the Arena. It moves randomly.
 *
 * 
 * @author Pavel Soukup 
 * @version v1
 */
public class Bee extends Actor
{ 
    private Beehive home;
    private int stepSize = 2;
    private GreenfootImage imageBasic = new GreenfootImage("bee.png");
    private int distanceFlown;
    private int urge = 1500+Greenfoot.getRandomNumber(501);
    private static int SMELL_DISTANCE = 50;
    private int smellDistance = SMELL_DISTANCE;
    private boolean hasFood = false;
    private boolean smellsFood = false;

    /**
     * Constructor for creating a beehive with orientation
     */
    public Bee(Beehive beehive){
        home = beehive;
        setScale();
        setRotation(Greenfoot.getRandomNumber(361));
    }

    /**
     * Act - do whatever the Ant wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    { 
        checkHome();
        if(hasFood){
            goTowards(home);
        } else {
            Actor food = smellFood();
            if(food!=null){
                goTowards(food);

                Actor closeFood;
                closeFood = getOneObjectAtOffset(0,0,Flower.class);

                if(closeFood!=null){
                    pickUpFood((Flower)closeFood);
                }
                
            } else {
                if(atWorldsEdge()){
                    turn(180);
                }else{
                    turn(10-Greenfoot.getRandomNumber(21)); //otoc nahodne v rozmezí -10 -> 10
                }
                move(stepSize);
            }
            distanceFlown++;
            urge--;
        if(Greenfoot.getRandomNumber(3000)>2998){
                    produceFood();

                }
            }
        }

    /**
     * Makes ant go in a direction.
     */
    public void goTowards(Actor target)
    {
        int x = target.getX();
        int y = target.getY();
        if(getX()==x&&getY()==y){
            //rest, you are there
        } else {
            turnTowards(x,y);
            move(stepSize);
        }
    }

    /**
     * Checks if ant is at home
     */
    public void checkHome()
    {
        if(Math.abs(getX() - home.getX()) <= stepSize && 
        Math.abs(getY() - home.getY()) <= stepSize){
            distanceFlown = 0;
            urge = urge + 1500+Greenfoot.getRandomNumber(801);
            if(hasFood == true){
                dropFood();
            }
        }
    }

    /**
     * Produce food
     *
     */
    public void produceFood()
    {
        getWorld().addObject(new HoneyDrop(20), getX(), getY());
    }

    /**
     * Return TRUE if ant is at the worlds edge
     */
    public boolean atWorldsEdge()
    {
        if(getX()< 12 ||getX() > getWorld().getWidth() - 12){ return true;}
        if(getY()< 12 ||getY() > getWorld().getHeight() - 12){ return true;}
        else{return false;}
    }

    /**
     * Return TRUE if ant is at the worlds edge
     */
    public void setScale()
    {
        GreenfootImage image = new GreenfootImage(imageBasic); //take original picture
        image.scale(25,25);
        setImage(image);
    }

    /**
     * An example of a method - replace this comment with your own
     *
     */
    public Actor smellFood()
    {
        Actor closest = null; //empty closest food reference
        if(distanceFlown > urge){
            goTowards(home);
        }
        else{
            if(!getObjectsInRange(smellDistance, Flower.class).isEmpty()){
                for(Object obj: getObjectsInRange(smellDistance, Flower.class)){
                    Actor food = (Actor) obj;
                    int foodDist = (int) Math.sqrt((Math.pow(food.getX()-getX(),2)+Math.pow(food.getY()-getY(),2)));

                    if(closest == null || foodDist < smellDistance){ //if distance is smaller than smellDistance OR there is NONE
                        closest = food;
                        smellDistance = foodDist; //decrease smell distance for next time to avoid smelling multiple sources
                    }
                }
            }
        }
        return(closest);
    }

    /**
     * An example of a method - replace this comment with your own
     */
    public void pickUpFood(Flower food)
    {
        hasFood = true;
    }

    /**
     * Drops food
     */
    public void dropFood()
    {
        hasFood = false;
        home.setFoodCount(home.getFoodCount()+1);
    }
}

